#include "PACKAGES_CONFIG.h"
#include "CPP_OPTIONS.h"

#define ALLOW_CPL_ISSM

CBOP
C     !ROUTINE: CPL_ISSM
C     !INTERFACE:
      SUBROUTINE CPL_ISSM( myTime, myIter, myThid )

C     !DESCRIPTION: \bv
C     *==================================================================
C     | SUBROUTINE cpl_issm
C     | o Couple MITgcm ocean model with ISSM ice sheet model
C     *==================================================================
C     \ev

C     !USES:
      IMPLICIT NONE
C     == Global variables ==
#include "SIZE.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "DYNVARS.h"
#include "GRID.h"
#include "FFIELDS.h"
#include "SHELFICE_OPTIONS.h"
#include "SHELFICE.h"
#ifdef ALLOW_EXF
# include "EXF_OPTIONS.h"
# include "EXF_FIELDS.h"
#endif

      LOGICAL  DIFFERENT_MULTIPLE
      EXTERNAL DIFFERENT_MULTIPLE

C     !LOCAL VARIABLES:
C     mytime - time counter for this thread (seconds)
C     myiter - iteration counter for this thread
C     mythid - thread number for this instance of the routine.
      _RL     mytime
      INTEGER myiter, mythid 
CEOP

#ifdef ALLOW_CPL_ISSM
#include "EESUPPORT.h"
      COMMON /CPL_MPI_ID/
     &     mpiMyWid, toissmcomm
      INTEGER mpiMyWid, toissmcomm, mpiRC
      integer mpistatus(MPI_STATUS_SIZE)
      real*8 CouplingTime, IceModelTime
      integer i, j, bi, bj, buffsize
      Real*8  xfer_array(Nx,Ny)
      _RL     local(1:sNx,1:sNy,nSx,nSy)

      IF( myTime .EQ. startTime ) THEN

C     Send deltatimestep
         _BEGIN_MASTER( myThid )
         call MPI_Recv(CouplingTime,1,MPI_DOUBLE,0,10001000,toissmcomm,
     &        mpistatus,mpiRC)
         print*, 'Ocean received CouplingTime: ', CouplingTime
         call MPI_Recv(IceModelTime,1,MPI_DOUBLE,0,10001001,toissmcomm,
     &        mpistatus,mpiRC)
         print*, 'Ocean received IceModelTime: ', IceModelTime
         call MPI_Send(myTime,1,MPI_DOUBLE,0,10001002,toissmcomm,
     &        mpistatus)
         call MPI_Send(Nx,1,MPI_INT,0,10001003,toissmcomm,
     &        mpistatus)
         call MPI_Send(Ny,1,MPI_INT,0,10001004,toissmcomm,
     &        mpistatus)
         _END_MASTER( myThid )

C     Send longitude East of center of cell
         DO bj=1,nSy
            DO bi=1,nSx
               DO j=1,sNy
                  DO i=1,sNx
                     local(i,j,bi,bj) = xC(i,j,bi,bj)
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
CDM         CALL GATHER_2D_R8( xfer_array, local, myThid )
         _BEGIN_MASTER( myThid )
         buffsize = Nx*Ny
         CALL MPI_SEND(xfer_array,buffsize,MPI_DOUBLE_PRECISION,
     &        0,10001005,toissmcomm,mpistatus)
         _END_MASTER( myThid )
         
C     Send latitude North of center of cell
         DO bj=1,nSy
            DO bi=1,nSx
               DO j=1,sNy
                  DO i=1,sNx
                     local(i,j,bi,bj) = yC(i,j,bi,bj)
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
CDM         CALL GATHER_2D_R8( xfer_array, local, myThid )
         _BEGIN_MASTER( myThid )
         buffsize = Nx*Ny
         CALL MPI_SEND(xfer_array,buffsize,MPI_DOUBLE_PRECISION,
     &        0,10001006,toissmcomm,mpistatus)
         _END_MASTER( myThid )

      ENDIF

      print*,'Done Sending XC/YC arrays.'

#endif /* ALLOW_CPL_MPMICE */

      RETURN
      END
